/*  app.h
 *
 *  The laserbreath application connects the ADC stream to the DAC stream
 *  and inserts a linear phase bandsplitter into the DAC stream.
 *  The sliders in the top half of the application window control
 *  the attenuation of each audio band and the bottom row of sliders
 *  controls a delay applied to each band.
 */

#include <Application.h>
#include <AudioSubscriber.h>

#include "slider.h"
#include "splitter.h"

struct LBApp : BApplication
{
  LBApp (int n_bands, float max_delay_secs);
  ~LBApp ();
  void AboutRequested();
  BAudioSubscriber* subscriber;
  Splitter* splitter;
  long processingRate;		// processing sample rate:  either
  							// the hardware rate or 1/2 the hardware rate
};

struct LBWindow : BWindow
{
  LBWindow (BRect r, char* name)
	: BWindow (r, name, B_TITLED_WINDOW, B_NOT_RESIZABLE | B_NOT_ZOOMABLE) {};
  bool QuitRequested() {be_app->PostMessage(B_QUIT_REQUESTED); return(TRUE);};
};

struct LBView : BView
{
  LBView (BRect r, char* name) : BView (r, name, B_FOLLOW_ALL, B_WILL_DRAW) {};
  void AttachedToWindow () {SetViewColor (30, 200, 130, 0);};
  void Draw (BRect);
};

struct GainSlider : LBSlider
{
  GainSlider (BRect r, char* name, int m) : LBSlider (r, name, m) {};
  void ValueChanged (float);
};

struct DelaySlider : LBSlider
{
  DelaySlider (BRect r, char* name, int m) : LBSlider (r, name, m) {};
  void ValueChanged (float);
};
