/*
  nino.cpp

  The Nino object copies buffers from the ADC stream to the DAC stream.
  It assumes the ADC and DAC buffers are the same size.
  See the Be Newsletter #36 for a detailed description.
*/

#include "nino.h"
#include <string.h>
#include <OS.h>

static bool adc_reader (void* arg, char* buf, long count)
{
  Nino* my_nino = (Nino*) arg;
  my_nino->buffer = buf;
  release_sem (my_nino->bufferReady);
  acquire_sem (my_nino->bufferUsed);
  return TRUE;
}

static bool dac_writer (void* arg, char* buf, long count)
{
  Nino* my_nino = (Nino*) arg;
  acquire_sem (my_nino->bufferReady);
  memcpy (buf, my_nino->buffer, count);
  release_sem (my_nino->bufferUsed);
  return TRUE;
}
  
Nino::Nino()
{
  bufferReady = create_sem (0, "buffer ready");
  bufferUsed = create_sem (0, "buffer used");
  adcSubscriber = new BAudioSubscriber ("Nino ADC reader");
  dacSubscriber = new BAudioSubscriber ("Nino DAC writer");
  adcSubscriber->Subscribe (B_ADC_STREAM, B_SHARED_SUBSCRIBER_ID, TRUE);
  dacSubscriber->Subscribe (B_DAC_STREAM, B_SHARED_SUBSCRIBER_ID, TRUE);
  adcSubscriber->EnterStream (NULL, FALSE, this, adc_reader, NULL, TRUE);
  dacSubscriber->EnterStream (NULL, TRUE, this, dac_writer, NULL, TRUE);
}

Nino::~Nino()
{
  dacSubscriber->ExitStream (TRUE);
  delete_sem (bufferReady);
  delete_sem (bufferUsed);
  adcSubscriber->ExitStream (TRUE);
  delete adcSubscriber;
  delete dacSubscriber;
}
