/*  Laserbreath -
 *    Rabiner / Gold development of recursive FFT Goertzel algorithm;
 *    linear phase bandsplitting combined with delays.
 *    Implements scheme described by Andy Moorer in JAES v34 #3.
 */

typedef double dsp_reg;			// type used for processing
typedef unsigned char bool;		// boolean type

// a frame is a stereo pair of samples

struct frame
{
  short L;
  short R;
};

static const frame zero_frame = {0, 0};

// each Band contains a comb filter and a recursive filter (resonator)

struct Band
{
  int delay;	// input delay in samples
  dsp_reg gain;	// nominally 1 for first and last bands, 2 for pole-pairs

  dsp_reg a1;	// recursive filter coefficients
  dsp_reg a2;
  dsp_reg yL;	// left channel recursive filter registers
  dsp_reg wL;
  dsp_reg yR;	// right channel recursive filter registers
  dsp_reg wR;

  frame* comb;	// comb filter delay line
};

// a Splitter contains an array of Bands and an input delay line

struct Splitter
{
  Splitter (int n_bands, int max_delay, bool halfspeed, double r = 0.999);
  virtual ~Splitter();

  void ProcessBuffer (frame* buf, int bufbytes);
  void SetGain (int m, float gain);
  void SetDelay (int m, int delay);

  Band* bands;		// bands
  frame* delayBuf;	// input delay line

  int maxDelay;		// size of input delay buffer
  int delayPointer;	// input delay buffer pointer
  int combPointer;	// comb filter delay line pointer

  int nBands;		// number of bands
  int nPoles;		// number of poles

  int prevL;		// previous output samples for interpolation
  int prevR;

  int downsampleFactor;	// 1 if full speed, 2 if half speed
};
